import os

USER_HOME_ELEANOR_CACHE = os.path.join(os.path.expanduser('~'), '.eleanor/')
USER_HOME_LIGHTKURVE = os.path.join(os.path.expanduser('~'), '.lightkurve-cache/')
LIGHTKURVE_CACHE_DIR = ".lightkurve-cache/"
ELEANOR_CACHE_DIR = ".eleanor/"
CUTOUT_SIZE = 11